@@ -28,8 +28,7 @@ import time
 from oslo_utils import excutils
 from oslo_utils import netutils
 import requests
-import six
-import six.moves.urllib.parse as urlparse
+import urllib.parse as urlparse
 from urllib3 import connection as httplib
 
 from oslo_vmware._i18n import _
@@ -135,7 +134,7 @@ class FileHandle(object):
                                self._build_vim_cookie_header(cookies)})
             if content_type:
                 headers.update({'Content-Type': content_type})
-            for key, value in six.iteritems(headers):
+            for key, value in headers.items():
                 conn.putheader(key, value)
             conn.endheaders()
             return conn
@@ -177,6 +176,32 @@ class FileHandle(object):
         """
         raise NotImplementedError()
 
+    def tell(self):
+        """Get the position of File Handle
+
+        :return: position
+        """
+        raise NotImplementedError()
+
+    def fileno(self):
+        """returns the integer file descriptor
+        by default this is not supported and raises IOError
+        """
+        raise IOError()
+
+    def seek(self, offset):
+        """sets the file's current position at the offset
+
+        :param offset: offset
+        """
+        pass
+
+    def flush(self):
+        """flushes the internal buffer
+
+        """
+        pass
+
     def get_size(self):
         """Get size of the file to be read.
 
@@ -328,7 +353,13 @@ class VmdkHandle(FileHandle):
         LOG.debug("Lease for %(url)s is in state: %(state)s.",
                   {'url': self._url,
                    'state': state})
-        if state == 'ready':
+        if self._get_progress() < 100:
+            LOG.error("Aborting lease for %s due to incomplete transfer.",
+                      self._url)
+            self._session.invoke_api(self._session.vim,
+                                     'HttpNfcLeaseAbort',
+                                     self._lease)
+        elif state == 'ready':
             LOG.debug("Releasing lease for %s.", self._url)
             self._session.invoke_api(self._session.vim,
                                      'HttpNfcLeaseComplete',
@@ -461,7 +492,7 @@ class VmdkWriteHandle(VmdkHandle):
         url, thumbprint = self._find_vmdk_url(lease_info, host, port)
         self._vm_ref = lease_info.entity
 
-        cookies = session.vim.client.options.transport.cookiejar
+        cookies = session.vim.client.cookiejar
         # Create HTTP connection to write to VMDK URL
         if http_method == 'PUT':
             overwrite = 't'
@@ -481,9 +512,19 @@ class VmdkWriteHandle(VmdkHandle):
         super(VmdkWriteHandle, self).__init__(session, lease, url, self._conn)
 
     def get_imported_vm(self):
-        """"Get managed object reference of the VM created for import."""
+        """"Get managed object reference of the VM created for import.
+
+        :raises: VimException
+        """
+        if self._get_progress() < 100:
+            excep_msg = _("Incomplete VMDK upload to %s.") % self._url
+            LOG.exception(excep_msg)
+            raise exceptions.ImageTransferException(excep_msg)
         return self._vm_ref
 
+    def tell(self):
+        return self._bytes_written
+
     def write(self, data):
         """Write data to the file.
 
@@ -559,13 +600,13 @@ class VmdkReadHandle(VmdkHandle):
 
         # find URL of the VMDK file to be read and open connection
         url, thumbprint = self._find_vmdk_url(lease_info, host, port)
-        cookies = session.vim.client.options.transport.cookiejar
+        cookies = session.vim.client.cookiejar
         self._conn = self._create_read_connection(url,
                                                   cookies=cookies,
                                                   ssl_thumbprint=thumbprint)
         super(VmdkReadHandle, self).__init__(session, lease, url, self._conn)
 
-    def read(self, chunk_size):
+    def read(self, chunk_size=READ_CHUNKSIZE):
         """Read a chunk of data from the VMDK file.
 
         :param chunk_size: size of read chunk
@@ -573,7 +614,7 @@ class VmdkReadHandle(VmdkHandle):
         :raises: VimException
         """
         try:
-            data = self._file_handle.read(READ_CHUNKSIZE)
+            data = self._file_handle.read(chunk_size)
             self._bytes_read += len(data)
             return data
         except Exception as excep:
@@ -585,6 +626,9 @@ class VmdkReadHandle(VmdkHandle):
             LOG.exception(excep_msg)
             raise exceptions.VimException(excep_msg, excep)
 
+    def tell(self):
+        return self._bytes_read
+
     def close(self):
         """Releases the lease and close the connection.
 
