@@ -1,4 +1,4 @@
-# Copyright (c) 2014 VMware, Inc.
+# Copyright (c) 2014-2020 VMware, Inc.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,15 +17,14 @@
 Common classes that provide access to vSphere services.
 """
 
+import http.client as httplib
+import io
 import logging
-import os
 
 import netaddr
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
 import requests
-import six
-import six.moves.http_client as httplib
 import suds
 from suds import cache
 from suds import client
@@ -96,7 +95,7 @@ class ServiceMessagePlugin(plugin.MessagePlugin):
         context.envelope.walk(self.add_attribute_for_value)
 
 
-class Response(six.BytesIO):
+class Response(io.BytesIO):
     """Response with an input stream as source."""
 
     def __init__(self, stream, status=200, headers=None):
@@ -104,7 +103,7 @@ class Response(six.BytesIO):
         self.headers = headers or {}
         self.reason = requests.status_codes._codes.get(
             status, [''])[0].upper().replace('_', ' ')
-        six.BytesIO.__init__(self, stream)
+        io.BytesIO.__init__(self, stream)
 
     @property
     def _original_response(self):
@@ -115,7 +114,7 @@ class Response(six.BytesIO):
         return self
 
     def read(self, chunk_size, **kwargs):
-        return six.BytesIO.read(self, chunk_size)
+        return io.BytesIO.read(self, chunk_size)
 
     def info(self):
         return self
@@ -144,9 +143,9 @@ class LocalFileAdapter(requests.adapters.HTTPAdapter):
 
     def _build_response_from_file(self, request):
         file_path = request.url[7:]
-        with open(file_path, 'r') as f:
-            buff = bytearray(os.path.getsize(file_path))
-            f.readinto(buff)
+        with open(file_path, 'rb') as f:
+            file_content = f.read()
+            buff = bytearray(file_content.decode(), "utf-8")
             resp = Response(buff)
             return self.build_response(request, resp)
 
@@ -173,7 +172,7 @@ class RequestsTransport(transport.Transport):
 
     def open(self, request):
         resp = self.session.get(request.url, verify=self.verify)
-        return six.BytesIO(resp.content)
+        return io.BytesIO(resp.content)
 
     def send(self, request):
         resp = self.session.post(request.url,
@@ -210,6 +209,25 @@ class MemoryCache(cache.ObjectCache):
 _CACHE = MemoryCache()
 
 
+class CompatibilitySudsClient(client.Client):
+    """suds client with added cookiejar attribute
+
+    The cookiejar properties allow reading/setting the cookiejar used by the
+    underlying transport.
+    """
+    def __init__(self, *args, **kwargs):
+        super(CompatibilitySudsClient, self).__init__(*args, **kwargs)
+
+    @property
+    def cookiejar(self):
+        return self.options.transport.cookiejar
+
+    @cookiejar.setter
+    def cookiejar(self, cookies):
+        self.options.transport.session.cookies = cookies
+        self.options.transport.cookiejar = cookies
+
+
 class Service(object):
     """Base class containing common functionality for invoking vSphere
     services
@@ -227,11 +245,11 @@ class Service(object):
                                       insecure=insecure,
                                       pool_maxsize=pool_maxsize,
                                       connection_timeout=connection_timeout)
-        self.client = client.Client(self.wsdl_url,
-                                    transport=transport,
-                                    location=self.soap_url,
-                                    plugins=[ServiceMessagePlugin()],
-                                    cache=_CACHE)
+        self.client = CompatibilitySudsClient(self.wsdl_url,
+                                              transport=transport,
+                                              location=self.soap_url,
+                                              plugins=[ServiceMessagePlugin()],
+                                              cache=_CACHE)
         self._service_content = None
         self._vc_session_cookie = None
 
@@ -275,7 +293,8 @@ class Service(object):
                         f_name = f_type.__class__.__name__
                         fault_list.append(f_name)
                         if f_name == exceptions.NO_PERMISSION:
-                            details['object'] = f_type.object.value
+                            details['object'] = \
+                                vim_util.get_moref_value(f_type.object)
                             details['privilegeId'] = f_type.privilegeId
 
         if fault_list:
@@ -312,7 +331,7 @@ class Service(object):
 
     def get_http_cookie(self):
         """Return the vCenter session cookie."""
-        cookies = self.client.options.transport.cookiejar
+        cookies = self.client.cookiejar
         for cookie in cookies:
             if cookie.name.lower() == 'vmware_soap_session':
                 return cookie.value
@@ -350,7 +369,7 @@ class Service(object):
                     op_id = '%s-%s' % (self.op_id_prefix,
                                        uuidutils.generate_uuid())
                     LOG.debug('Invoking %s.%s with opID=%s',
-                              managed_object._type,
+                              vim_util.get_moref_type(managed_object),
                               attr_name,
                               op_id)
                 self._set_soap_headers(op_id)
@@ -414,13 +433,13 @@ class Service(object):
 
                 # Socket errors which need special handling; some of these
                 # might be caused by server API call overload.
-                if (six.text_type(excep).find(ADDRESS_IN_USE_ERROR) != -1 or
-                        six.text_type(excep).find(CONN_ABORT_ERROR)) != -1:
+                if (str(excep).find(ADDRESS_IN_USE_ERROR) != -1 or
+                        str(excep).find(CONN_ABORT_ERROR)) != -1:
                     raise exceptions.VimSessionOverLoadException(
                         _("Socket error in %s.") % attr_name, excep)
                 # Type error which needs special handling; it might be caused
                 # by server API call overload.
-                elif six.text_type(excep).find(RESP_NOT_XML_ERROR) != -1:
+                elif str(excep).find(RESP_NOT_XML_ERROR) != -1:
                     raise exceptions.VimSessionOverLoadException(
                         _("Type error in %s.") % attr_name, excep)
                 else:
@@ -460,5 +479,6 @@ class SudsLogFilter(logging.Filter):
 
         return True
 
+
 # Set log filter to mask/truncate vCenter credentials in suds logs.
 suds.client.log.addFilter(SudsLogFilter())
