@@ -1,4 +1,4 @@
-# Copyright (c) 2014 VMware, Inc.
+# Copyright (c) 2014-2020 VMware, Inc.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,10 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+import http.client as httplib
+import io
+from unittest import mock
+
+import ddt
 import requests
-import six
-import six.moves.http_client as httplib
 import suds
 
 from oslo_vmware import exceptions
@@ -25,30 +27,40 @@ from oslo_vmware.tests import base
 from oslo_vmware import vim_util
 
 
+@ddt.ddt
 class ServiceMessagePluginTest(base.TestCase):
     """Test class for ServiceMessagePlugin."""
 
-    def test_add_attribute_for_value(self):
+    def setUp(self):
+        super(ServiceMessagePluginTest, self).setUp()
+        self.plugin = service.ServiceMessagePlugin()
+
+    @ddt.data(('value', 'foo', 'string'), ('removeKey', '1', 'int'),
+              ('removeKey', 'foo', 'string'))
+    @ddt.unpack
+    def test_add_attribute_for_value(self, name, text, expected_xsd_type):
         node = mock.Mock()
-        node.name = 'value'
-        plugin = service.ServiceMessagePlugin()
-        plugin.add_attribute_for_value(node)
-        node.set.assert_called_once_with('xsi:type', 'xsd:string')
+        node.name = name
+        node.text = text
+        self.plugin.add_attribute_for_value(node)
+        node.set.assert_called_once_with('xsi:type',
+                                         'xsd:%s' % expected_xsd_type)
 
     def test_marshalled(self):
-        plugin = service.ServiceMessagePlugin()
         context = mock.Mock()
-        plugin.marshalled(context)
-        context.envelope.prune.assert_called_once_with()
+        self.plugin.prune = mock.Mock()
+        self.plugin.marshalled(context)
+        self.plugin.prune.assert_called_once_with(context.envelope)
         context.envelope.walk.assert_called_once_with(
-            plugin.add_attribute_for_value)
+            self.plugin.add_attribute_for_value)
 
 
+@ddt.ddt
 class ServiceTest(base.TestCase):
 
     def setUp(self):
         super(ServiceTest, self).setUp()
-        patcher = mock.patch('suds.client.Client')
+        patcher = mock.patch('oslo_vmware.service.CompatibilitySudsClient')
         self.addCleanup(patcher.stop)
         self.SudsClientMock = patcher.start()
 
@@ -83,8 +95,8 @@ class ServiceTest(base.TestCase):
         resp = mock.Mock()
 
         def side_effect(mo, **kwargs):
-            self.assertEqual(managed_object, mo._type)
-            self.assertEqual(managed_object, mo.value)
+            self.assertEqual(managed_object, vim_util.get_moref_type(mo))
+            self.assertEqual(managed_object, vim_util.get_moref_value(mo))
             return resp
 
         svc_obj = service.Service()
@@ -98,8 +110,8 @@ class ServiceTest(base.TestCase):
         managed_object = 'Datacenter'
 
         def side_effect(mo, **kwargs):
-            self.assertEqual(managed_object, mo._type)
-            self.assertEqual(managed_object, mo.value)
+            self.assertEqual(managed_object, vim_util.get_moref_type(mo))
+            self.assertEqual(managed_object, vim_util.get_moref_value(mo))
             return None
 
         svc_obj = service.Service()
@@ -117,8 +129,8 @@ class ServiceTest(base.TestCase):
         doc = mock.Mock()
 
         def side_effect(mo, **kwargs):
-            self.assertEqual(managed_object, mo._type)
-            self.assertEqual(managed_object, mo.value)
+            self.assertEqual(managed_object, vim_util.get_moref_type(mo))
+            self.assertEqual(managed_object, vim_util.get_moref_value(mo))
             fault = mock.Mock(faultstring="MyFault")
 
             fault_children = mock.Mock()
@@ -168,8 +180,8 @@ class ServiceTest(base.TestCase):
         doc = mock.Mock()
 
         def side_effect(mo, **kwargs):
-            self.assertEqual(managed_object, mo._type)
-            self.assertEqual(managed_object, mo.value)
+            self.assertEqual(managed_object, vim_util.get_moref_type(mo))
+            self.assertEqual(managed_object, vim_util.get_moref_value(mo))
             fault = mock.Mock(faultstring="MyFault")
 
             fault_children = mock.Mock()
@@ -197,20 +209,21 @@ class ServiceTest(base.TestCase):
                      mock.call('/Envelope/Body/Fault/detail')]
         self.assertEqual(exp_calls, doc.childAtPath.call_args_list)
 
-    def test_request_handler_with_security_error(self):
-        managed_object = 'VirtualMachine'
+    @ddt.data('vim25:SecurityError', 'vim25:NotAuthenticated')
+    def test_request_handler_with_pbm_session_error(self, fault_name):
+        managed_object = 'ProfileManager'
         doc = mock.Mock()
 
         def side_effect(mo, **kwargs):
-            self.assertEqual(managed_object, mo._type)
-            self.assertEqual(managed_object, mo.value)
+            self.assertEqual(managed_object, vim_util.get_moref_type(mo))
+            self.assertEqual(managed_object, vim_util.get_moref_value(mo))
             fault = mock.Mock(faultstring="MyFault")
 
             fault_children = mock.Mock()
             fault_children.name = "name"
             fault_children.getText.return_value = "value"
             child = mock.Mock()
-            child.get.return_value = 'vim25:SecurityError'
+            child.get.return_value = fault_name
             child.getChildren.return_value = [fault_children]
             detail = mock.Mock()
             detail.getChildren.return_value = [child]
@@ -219,9 +232,10 @@ class ServiceTest(base.TestCase):
 
         svc_obj = service.Service()
         service_mock = svc_obj.client.service
-        setattr(service_mock, 'powerOn', side_effect)
+        setattr(service_mock, 'get_profile_id_by_name', side_effect)
 
-        ex = self.assertRaises(exceptions.VimFaultException, svc_obj.powerOn,
+        ex = self.assertRaises(exceptions.VimFaultException,
+                               svc_obj.get_profile_id_by_name,
                                managed_object)
 
         self.assertEqual([exceptions.NOT_AUTHENTICATED], ex.fault_list)
@@ -243,8 +257,8 @@ class ServiceTest(base.TestCase):
         managed_object = 'VirtualMachine'
 
         def side_effect(mo, **kwargs):
-            self.assertEqual(managed_object, mo._type)
-            self.assertEqual(managed_object, mo.value)
+            self.assertEqual(managed_object, vim_util.get_moref_type(mo))
+            self.assertEqual(managed_object, vim_util.get_moref_value(mo))
             raise httplib.CannotSendRequest()
 
         svc_obj = service.Service()
@@ -259,8 +273,8 @@ class ServiceTest(base.TestCase):
         managed_object = 'VirtualMachine'
 
         def side_effect(mo, **kwargs):
-            self.assertEqual(managed_object, mo._type)
-            self.assertEqual(managed_object, mo.value)
+            self.assertEqual(managed_object, vim_util.get_moref_type(mo))
+            self.assertEqual(managed_object, vim_util.get_moref_value(mo))
             raise httplib.ResponseNotReady()
 
         svc_obj = service.Service()
@@ -275,8 +289,8 @@ class ServiceTest(base.TestCase):
         managed_object = 'VirtualMachine'
 
         def side_effect(mo, **kwargs):
-            self.assertEqual(managed_object, mo._type)
-            self.assertEqual(managed_object, mo.value)
+            self.assertEqual(managed_object, vim_util.get_moref_type(mo))
+            self.assertEqual(managed_object, vim_util.get_moref_value(mo))
             raise httplib.CannotSendHeader()
 
         svc_obj = service.Service()
@@ -291,8 +305,8 @@ class ServiceTest(base.TestCase):
         managed_object = 'VirtualMachine'
 
         def side_effect(mo, **kwargs):
-            self.assertEqual(managed_object, mo._type)
-            self.assertEqual(managed_object, mo.value)
+            self.assertEqual(managed_object, vim_util.get_moref_type(mo))
+            self.assertEqual(managed_object, vim_util.get_moref_value(mo))
             raise requests.ConnectionError()
 
         svc_obj = service.Service()
@@ -307,8 +321,8 @@ class ServiceTest(base.TestCase):
         managed_object = 'VirtualMachine'
 
         def side_effect(mo, **kwargs):
-            self.assertEqual(managed_object, mo._type)
-            self.assertEqual(managed_object, mo.value)
+            self.assertEqual(managed_object, vim_util.get_moref_type(mo))
+            self.assertEqual(managed_object, vim_util.get_moref_value(mo))
             raise requests.HTTPError()
 
         svc_obj = service.Service()
@@ -330,8 +344,8 @@ class ServiceTest(base.TestCase):
         managed_object = 'VirtualMachine'
 
         def side_effect(mo, **kwargs):
-            self.assertEqual(managed_object, mo._type)
-            self.assertEqual(managed_object, mo.value)
+            self.assertEqual(managed_object, vim_util.get_moref_type(mo))
+            self.assertEqual(managed_object, vim_util.get_moref_value(mo))
             raise Exception(message)
 
         svc_obj = service.Service()
@@ -363,7 +377,7 @@ class ServiceTest(base.TestCase):
         cookie = mock.Mock()
         cookie.name = 'vmware_soap_session'
         cookie.value = cookie_value
-        svc_obj.client.options.transport.cookiejar = [cookie]
+        svc_obj.client.cookiejar = [cookie]
         self.assertEqual(cookie_value, svc_obj.get_http_cookie())
 
     def test_get_session_cookie_with_no_cookie(self):
@@ -371,9 +385,33 @@ class ServiceTest(base.TestCase):
         cookie = mock.Mock()
         cookie.name = 'cookie'
         cookie.value = 'xyz'
-        svc_obj.client.options.transport.cookiejar = [cookie]
+        svc_obj.client.cookiejar = [cookie]
         self.assertIsNone(svc_obj.get_http_cookie())
 
+    def test_set_soap_headers(self):
+        def fake_set_options(*args, **kwargs):
+            headers = kwargs['soapheaders']
+            self.assertEqual(1, len(headers))
+            txt = headers[0].getText()
+            self.assertEqual('fira-12345', txt)
+
+        svc_obj = service.Service()
+        svc_obj.client.options.soapheaders = None
+        setattr(svc_obj.client, 'set_options', fake_set_options)
+        svc_obj._set_soap_headers('fira-12345')
+
+    def test_soap_headers_pbm(self):
+        def fake_set_options(*args, **kwargs):
+            headers = kwargs['soapheaders']
+            self.assertEqual(2, len(headers))
+            self.assertEqual('vc-session-cookie', headers[0].getText())
+            self.assertEqual('fira-12345', headers[1].getText())
+
+        svc_obj = service.Service()
+        svc_obj._vc_session_cookie = 'vc-session-cookie'
+        setattr(svc_obj.client, 'set_options', fake_set_options)
+        svc_obj._set_soap_headers('fira-12345')
+
 
 class MemoryCacheTest(base.TestCase):
     """Test class for MemoryCache."""
@@ -386,8 +424,9 @@ class MemoryCacheTest(base.TestCase):
         self.assertEqual('value2', cache.get('key2'))
         self.assertIsNone(cache.get('key3'))
 
-    @mock.patch('suds.reader.DocumentReader.download')
-    def test_shared_cache(self, mock_reader):
+    @mock.patch('suds.reader.DefinitionsReader.open')
+    @mock.patch('suds.reader.DocumentReader.download', create=True)
+    def test_shared_cache(self, mock_reader, mock_open):
         cache1 = service.Service().client.options.cache
         cache2 = service.Service().client.options.cache
         self.assertIs(cache1, cache2)
@@ -445,6 +484,9 @@ class RequestsTransportTest(base.TestCase):
         local_file_adapter = transport.session.adapters['file:///']
         self.assertEqual(100, local_file_adapter._pool_connections)
         self.assertEqual(100, local_file_adapter._pool_maxsize)
+        https_adapter = transport.session.adapters['https://']
+        self.assertEqual(100, https_adapter._pool_connections)
+        self.assertEqual(100, https_adapter._pool_maxsize)
 
     @mock.patch('os.path.getsize')
     def test_send_with_local_file_url(self, get_size_mock):
@@ -457,32 +499,40 @@ class RequestsTransportTest(base.TestCase):
         data = b"Hello World"
         get_size_mock.return_value = len(data)
 
-        def readinto_mock(buf):
-            buf[0:] = data
+        def read_mock():
+            return data
 
-        if six.PY3:
-            builtin_open = 'builtins.open'
-            open_mock = mock.MagicMock(name='file_handle',
-                                       spec=open)
-            import _io
-            file_spec = list(set(dir(_io.TextIOWrapper)).union(
-                set(dir(_io.BytesIO))))
-        else:
-            builtin_open = '__builtin__.open'
-            open_mock = mock.MagicMock(name='file_handle',
-                                       spec=file)
-            file_spec = file
+        builtin_open = 'builtins.open'
+        open_mock = mock.MagicMock(name='file_handle',
+                                   spec=open)
+        file_spec = list(set(dir(io.TextIOWrapper)).union(
+            set(dir(io.BytesIO))))
 
         file_handle = mock.MagicMock(spec=file_spec)
         file_handle.write.return_value = None
         file_handle.__enter__.return_value = file_handle
-        file_handle.readinto.side_effect = readinto_mock
+        file_handle.read.side_effect = read_mock
         open_mock.return_value = file_handle
 
         with mock.patch(builtin_open, open_mock, create=True):
             resp = transport.session.send(request)
             self.assertEqual(data, resp.content)
 
+    def test_send_with_connection_timeout(self):
+        transport = service.RequestsTransport(connection_timeout=120)
+
+        request = mock.Mock(url=mock.sentinel.url,
+                            message=mock.sentinel.message,
+                            headers=mock.sentinel.req_headers)
+        with mock.patch.object(transport.session, "post") as mock_post:
+            transport.send(request)
+            mock_post.assert_called_once_with(
+                mock.sentinel.url,
+                data=mock.sentinel.message,
+                headers=mock.sentinel.req_headers,
+                timeout=120,
+                verify=transport.verify)
+
 
 class SudsLogFilterTest(base.TestCase):
     """Tests for SudsLogFilter."""
