@@ -22,9 +22,6 @@ import logging
 from oslo_utils import timeutils
 from suds import sudsobject
 
-from oslo_vmware._i18n import _LW
-
-
 LOG = logging.getLogger(__name__)
 
 
@@ -40,6 +37,44 @@ def get_moref(value, type_):
     return moref
 
 
+def get_moref_value(moref):
+    """Get the value/id of a managed object reference
+
+    This function accepts a string representation of a ManagedObjectReference
+    like `VirtualMachine:vm-123` or only `vm-123`, but is also able to extract
+    it from the actual object as returned by the API.
+    """
+    if isinstance(moref, str):
+        # handle strings like VirtualMachine:vm-12312, but also vm-123123
+        if ':' in moref:
+            splits = moref.split(':')
+            return splits[1]
+        return moref
+
+    # assume it's a ManagedObjectReference object as created by `get_moref()`
+    # or returned by a request
+    return moref.value
+
+
+def get_moref_type(moref):
+    """Get the type of a managed object reference
+
+    This function accepts a string representation of a ManagedObjectReference
+    like `VirtualMachine:vm-123`, but is also able to extract it from the
+    actual object as returned by the API.
+    """
+    if isinstance(moref, str):
+        # handle strings like VirtualMachine:vm-12312
+        if ':' in moref:
+            splits = moref.split(':')
+            return splits[0]
+        return None
+
+    # assume it's a ManagedObjectReference object as created by `get_moref()`
+    # or returned by a request
+    return moref._type
+
+
 def build_selection_spec(client_factory, name):
     """Builds the selection spec.
 
@@ -100,6 +135,12 @@ def build_recursive_traversal_spec(client_factory):
                                       'networkFolder',
                                       False,
                                       [visit_folders_select_spec])
+    dc_to_df = build_traversal_spec(client_factory,
+                                    'dc_to_df',
+                                    'Datacenter',
+                                    'datastoreFolder',
+                                    False,
+                                    [visit_folders_select_spec])
 
     # Next hop from HostSystem
     h_to_vm = build_traversal_spec(client_factory,
@@ -182,6 +223,7 @@ def build_recursive_traversal_spec(client_factory):
                                            dc_to_hf,
                                            dc_to_vmf,
                                            dc_to_netf,
+                                           dc_to_df,
                                            cr_to_ds,
                                            cr_to_h,
                                            cr_to_rp,
@@ -285,13 +327,14 @@ def get_objects(vim, type_, max_objects, properties_to_collect=None,
                                     options=options)
 
 
-def get_object_properties(vim, moref, properties_to_collect):
+def get_object_properties(vim, moref, properties_to_collect, skip_op_id=False):
     """Get properties of the given managed object.
 
     :param vim: Vim object
     :param moref: managed object reference
     :param properties_to_collect: names of the managed object properties to be
                                   collected
+    :param skip_op_id: whether to skip putting opID in the request
     :returns: properties of the given managed object
     :raises: VimException, VimFaultException, VimAttributeException,
              VimSessionOverLoadException, VimConnectionException
@@ -304,7 +347,7 @@ def get_object_properties(vim, moref, properties_to_collect):
                       len(properties_to_collect) == 0)
     property_spec = build_property_spec(
         client_factory,
-        type_=moref._type,
+        type_=get_moref_type(moref),
         properties_to_collect=properties_to_collect,
         all_properties=all_properties)
     object_spec = build_object_spec(client_factory, moref, [])
@@ -317,7 +360,8 @@ def get_object_properties(vim, moref, properties_to_collect):
     retrieve_result = vim.RetrievePropertiesEx(
         vim.service_content.propertyCollector,
         specSet=[property_filter_spec],
-        options=options)
+        options=options,
+        skip_op_id=skip_op_id)
     cancel_retrieval(vim, retrieve_result)
     return retrieve_result.objects
 
@@ -345,8 +389,8 @@ def get_object_properties_dict(vim, moref, properties_to_collect):
     # The object may have information useful for logging
     if hasattr(obj_contents[0], 'missingSet'):
         for m in obj_contents[0].missingSet:
-            LOG.warning(_LW("Unable to retrieve value for %(path)s "
-                            "Reason: %(reason)s"),
+            LOG.warning("Unable to retrieve value for %(path)s "
+                        "Reason: %(reason)s",
                         {'path': m.path,
                          'reason': m.fault.localizedMessage})
     return property_dict
@@ -422,17 +466,19 @@ class WithRetrieval(object):
                 self.vim, self.retrieve_result)
 
 
-def get_object_property(vim, moref, property_name):
+def get_object_property(vim, moref, property_name, skip_op_id=False):
     """Get property of the given managed object.
 
     :param vim: Vim object
     :param moref: managed object reference
     :param property_name: name of the property to be retrieved
+    :param skip_op_id: whether to skip putting opID in the request
     :returns: property of the given managed object
     :raises: VimException, VimFaultException, VimAttributeException,
              VimSessionOverLoadException, VimConnectionException
     """
-    props = get_object_properties(vim, moref, [property_name])
+    props = get_object_properties(vim, moref, [property_name],
+                                  skip_op_id=skip_op_id)
     prop_val = None
     if props:
         prop = None
@@ -453,12 +499,12 @@ def find_extension(vim, key):
     :returns: the data object Extension or None
     """
     extension_manager = vim.service_content.extensionManager
-    return vim.client.service.FindExtension(extension_manager, key)
+    return vim.FindExtension(extension_manager, extensionKey=key)
 
 
 def register_extension(vim, key, type, label='OpenStack',
                        summary='OpenStack services', version='1.0'):
-    """Create a new extention.
+    """Create a new extension.
 
     :param vim: Vim object
     :param key: the key for the extension
@@ -482,7 +528,7 @@ def register_extension(vim, key, type, label='OpenStack',
     desc.summary = summary
     os_ext.description = desc
     os_ext.lastHeartbeatTime = timeutils.utcnow().isoformat()
-    vim.client.service.RegisterExtension(extension_manager, os_ext)
+    vim.RegisterExtension(extension_manager, extension=os_ext)
 
 
 def get_vc_version(session):
@@ -552,3 +598,99 @@ def get_http_service_request_spec(client_factory, method, uri):
     http_service_request_spec.method = method
     http_service_request_spec.url = uri
     return http_service_request_spec
+
+
+def get_prop_spec(client_factory, spec_type, properties):
+    """Builds the Property Spec Object."""
+    prop_spec = client_factory.create('ns0:PropertySpec')
+    prop_spec.type = spec_type
+    prop_spec.pathSet = properties
+    return prop_spec
+
+
+def get_obj_spec(client_factory, obj, select_set=None):
+    """Builds the Object Spec object."""
+    obj_spec = client_factory.create('ns0:ObjectSpec')
+    obj_spec.obj = obj
+    obj_spec.skip = False
+    if select_set is not None:
+        obj_spec.selectSet = select_set
+    return obj_spec
+
+
+def get_prop_filter_spec(client_factory, obj_spec, prop_spec):
+    """Builds the Property Filter Spec Object."""
+    prop_filter_spec = client_factory.create('ns0:PropertyFilterSpec')
+    prop_filter_spec.propSet = prop_spec
+    prop_filter_spec.objectSet = obj_spec
+    return prop_filter_spec
+
+
+def get_properties_for_a_collection_of_objects(vim, type_,
+                                               obj_list, properties,
+                                               max_objects=None):
+    """Gets the list of properties for the collection of
+    objects of the type specified.
+    """
+    client_factory = vim.client.factory
+    if len(obj_list) == 0:
+        return []
+    prop_spec = get_prop_spec(client_factory, type_, properties)
+    lst_obj_specs = []
+    for obj in obj_list:
+        lst_obj_specs.append(get_obj_spec(client_factory, obj))
+    prop_filter_spec = get_prop_filter_spec(client_factory,
+                                            lst_obj_specs, [prop_spec])
+    options = client_factory.create('ns0:RetrieveOptions')
+    options.maxObjects = max_objects if max_objects else len(obj_list)
+    return vim.RetrievePropertiesEx(
+        vim.service_content.propertyCollector,
+        specSet=[prop_filter_spec], options=options)
+
+
+def propset_dict(propset):
+    """Turn a propset list into a dictionary
+
+    PropSet is an optional attribute on ObjectContent objects
+    that are returned by the VMware API.
+
+    You can read more about these at:
+    | http://pubs.vmware.com/vsphere-51/index.jsp
+    |    #com.vmware.wssdk.apiref.doc/
+    |        vmodl.query.PropertyCollector.ObjectContent.html
+
+    :param propset: a property "set" from ObjectContent
+    :return: dictionary representing property set
+    """
+    if propset is None:
+        return {}
+
+    return {prop.name: prop.val for prop in propset}
+
+
+def storage_placement_spec(client_factory,
+                           dsc_ref,
+                           type,
+                           clone_spec=None,
+                           config_spec=None,
+                           relocate_spec=None,
+                           vm_ref=None,
+                           folder=None,
+                           clone_name=None,
+                           res_pool_ref=None,
+                           host_ref=None):
+    pod_sel_spec = client_factory.create('ns0:StorageDrsPodSelectionSpec')
+    pod_sel_spec.storagePod = dsc_ref
+
+    spec = client_factory.create('ns0:StoragePlacementSpec')
+    spec.podSelectionSpec = pod_sel_spec
+    spec.type = type
+    spec.vm = vm_ref
+    spec.folder = folder
+    spec.cloneSpec = clone_spec
+    spec.configSpec = config_spec
+    spec.relocateSpec = relocate_spec
+    spec.cloneName = clone_name
+    spec.resourcePool = res_pool_ref
+    spec.host = host_ref
+    return spec
